from threading import Thread
from time import sleep
from sys import stdout

from .colors import Color, Fade
from .pattern import Pattern
from .system import sys
from .ansi import CLEAR, RESET


__all__ = ['FadeAnimation', 'PatternAnimation', 'ProgressBar', 'smooth_show']


class FadeAnimation:
    banner: str
    """The text to be animated."""
    fade: Fade
    """The fade that move over the text"""

    __slots__ = ('banner', 'fade')

    def __init__(self,
                 banner: str,
                 fade: Fade) -> None:
        """
        Create an animation of a fade mooving on a text.

        ### arguments
         - `banner`: The text to be animated.
         - `fade`: The fade that move over the text
        """
        self.banner = banner
        self.fade = fade


    def animate(self,
                duration: float | None = None,
                interval: float = 0.05) -> None:
        """
        Animate the animation.

        ### arguments
         - `duration?`: The duration of the animations in seconds.
         - `interval?`: The interval between each frame of the animation in seconds.
        """
        passed = False
        cursor = sys.cursor
        sys.cursor = False

        def wait():
            nonlocal passed
            if duration:
                sleep(float(duration))
            else:
                input()
            passed = True

        Thread(target=wait).start()
        decal = 0

        while not passed:
            colored = self.fade.colorate(self.banner, decal)
            stdout.write(CLEAR + colored)
            decal += 1
            sleep(interval)

        sys.clear()
        sys.cursor = cursor


    __call__ = animate


class PatternAnimation:
    banner: str
    """The text to be animated."""
    pattern: Pattern
    """The pattern that animate"""

    __slots__ = ('banner', 'pattern')

    def __init__(self,
                 banner: str,
                 pattern: Pattern) -> None:
        """
        Create an animation of a pattern with the fades mooving on a text.

        ### arguments
         - `banner`: The text to be animated.
         - `pattern`: The pattern that animate
        """
        self.banner = banner
        self.pattern = pattern


    def animate(self,
                duration: float | None = None,
                interval: float = 0.05) -> None:
        """
        Animate the animation.

        ### arguments
         - `duration?`: The duration of the animations in seconds.
         - `interval?`: The interval between each frame of the animation in seconds.
        """
        passed = False
        cursor = sys.cursor
        sys.cursor = False

        def wait():
            nonlocal passed
            if duration:
                sleep(float(duration))
            else:
                input()
            passed = True

        Thread(target=wait).start()
        decal = 0

        while not passed:
            colored = self.pattern.colorate(self.banner, decal)
            stdout.write(CLEAR + colored)
            decal += 1
            sleep(interval)

        sys.clear()
        sys.cursor = cursor
        stdout.write(RESET)


    __call__ = animate


class ProgressBar:
    name: str
    """The name of the progress bar"""
    steps: int
    """The number of steps the progress bar can reach"""
    color: Color | Fade
    """The color or fade the progress bar will have on it reached part"""
    length: int
    """The length of the progress bar"""
    step: int
    """The current step where the progress bar is"""
    step_comment: str | None
    """The text that is displayed next to the bar to show a comment"""

    __slots__ = ('name', 'steps', 'color',
                 'length', 'step', 'step_comment')

    def __init__(self,
                 name: str,
                 steps: int,
                 color: Fade,
                 length: int = 20) -> None:
        """
        Create a progress bar that can increment and show comments.

        ### arguments
         - `name`: The name of the progress bar.
         - `steps`: The number of steps the progress bar can reach.
         - `color`: The color or fade the progress bar will have on it reached part.
         - `length?`: The length of the progress bar.
        """
        self.name = name
        self.steps = steps
        self.color = color
        self.length = length
        self.step = 0
        self.step_comment = None


    def increment(self,
                  steps: int = 1,
                  comment: str | None = None) -> None:
        """
        Increment the progress.

        ### arguments
         - `steps?`: The number of steps to increment
         - `comment?`: An optional comment to show next to the bar
        """
        self.step += steps
        self.step_comment = comment


    def show(self,
             interval: float = 0.05,
             lpadding: int = 0) -> None:
        """
        Show the progress bar.

        ### arguments
         - `interval?`: The interval between each reload of the progress bar in seconds.
         - `lpadding?`: The left padding margin of the progress bar.
        """
        cursor = sys.cursor
        sys.cursor = False

        def _show():
            if isfade := isinstance(self.color, Fade):
                colcount = len(self.color.getcolors()) * 2
                decal = colcount

            while True:
                progress = self.step / self.steps
                bar_length = int(self.length * progress)

                reached = '█' * bar_length
                if isfade:
                    colored_reached = self.color.colorate(reached, decal) + RESET
                    decal -= 1
                    if decal <= 0:
                        decal = colcount
                else:
                    colored_reached = self.color + reached

                rest = Color(25, 25, 25) + ('█' * (self.length - bar_length))
                comment = self.step_comment or ''
                stats = Color(100, 100, 100) + f'{self.step}/{self.steps}'
                bar = self.name + ' ' + colored_reached + rest + ' ' + stats + ' ' +  comment

                stdout.write('\r' + (' ' * lpadding) + bar + RESET)

                if self.step >= self.steps:
                    break
                sleep(interval)

            sys.cursor = cursor
            stdout.write(RESET)

        Thread(target=_show).start()


    __call__ = show


def smooth_show(wait: bool = False,
                interval: float = 0.005,
                transparency: int = 255) -> None:
    """
    Show the terminal smoothly with its transparency.

    ### arguments
     - `wait?`: If the fonction should wait the terminal to have  the given final opacity.
     - `interval?`: The interval between each transparency update in seconds.
     - `transparency?`: The final transparency
    """
    def show():
        for i in range(transparency + 1):
            sys.transparency = i
            sleep(interval)

    if wait:
        show()
    else:
        Thread(target=show).start()


import ctypes
from os import name, system
from sys import stdout, exc_info

from .ansi import *


__all__ = ['sys']



class System:
    windows: bool
    """If the current operating system is Windows"""
    clear_at_end: bool
    """If the system should clear the terminal when the script stop"""
    
    def __init__(self) -> None:
        self.windows = name == 'nt'
        self.clear_at_end = False
        self.__cursor = True
        self.__title = ''
        self.__transparency = -1

        if self.windows:
            self.__console = ctypes.windll.kernel32.GetConsoleWindow()
        else:
            self.__console = None


    def __del__(self) -> None:
        if self.clear_at_end:
            self.clear()

        stdout.write(RESET)
        self.cursor = True


    @property
    def title(self) -> str:
        """The title of the terminal"""
        return self.__title


    @property
    def size(self) -> tuple[int, int]:
        """The size of the terminal (x, y)"""
        return self._size


    @property
    def cursor(self) -> bool:
        """if the cursor is activated in the terminal"""
        return self.__cursor


    @property
    def transparency(self) -> int:
        """The transparency of the terminal in byte"""
        return self.__transparency


    @title.setter
    def title(self, value: str) -> None:
        stdout.write(TITLE.format(value))
        self.__title = value


    @size.setter
    def size(self, value: tuple[int, int]) -> None:
        if self.windows:
            system('mode {}, {}'.format(*value))
            self._size = value


    @cursor.setter
    def cursor(self, value: bool) -> None:
        stdout.write(CURSOR if value else NO_CURSOR)
        self.__cursor = value


    @transparency.setter
    def transparency(self, value: int) -> None:
        if self.windows:
            ctypes.windll.user32.SetLayeredWindowAttributes(self.__console, 0, value, 2)
            self.__transparency = value


    def clear(self) -> None:
        """
        Clear all the text in the terminal.
        """
        stdout.write(CLEAR)
        stdout.flush()


sys: System = System()


from typing import LiteralString


__all__ = ['FG_COLOR', 'BG_COLOR', 'RESET',
           'CLEAR', 'TITLE', 'CURSOR',
           'NO_CURSOR', 'UNDERLINE']

FG_COLOR: LiteralString = '\x1b[38;2;{};{};{}m'
"""The ANSI sequence to colorate text with the RGB color value"""
BG_COLOR: LiteralString = '\x1b[48;2;{};{};{}m'
"""The ANSI sequence to colorate the background with the RGB color value"""
RESET: LiteralString = '\x1b[0m'
"""The ANSI sequence to reset all the styles"""
CLEAR: LiteralString = '\x1b[2J\x1b[H'
"""The ANSI sequence to clear all the text before the sequence"""
TITLE: LiteralString = '\x1b]O;{}\a'
"""The ANSI sequence to change the terminal title"""
CURSOR: LiteralString = '\x1b[?25h'
"""The ANSI sequence to activate the terminal cursor"""
NO_CURSOR: LiteralString = '\x1b[?25l'
"""The ANSI sequence to desactivate the terminal cursor"""
UNDERLINE: LiteralString = '\x1b[4m'
"""The ANSI sequence to underline a text"""


from os import system as _system
from importlib.metadata import version

from .system import *
from .colors import *
from .align import *
from .pattern import *
from .animations import *
from .display import *


__all__ = [obj for obj in globals() if not obj.startswith('_')]
__version__ = version(__name__)
__doc__ = """
ZxFade is a Python package to make your programs easy to use and nice to look at.
It contains colors, fades, patterns, animations, system, and more.
by BlueRed.
"""

_system('')


from os import get_terminal_size

from .colors import rmansi
from .types import *


__all__ = ['ycenter', 'xcenter', 'center']


def ycenter(text: AnyStringT,
            size: int | None = None) -> AnyStringT:
    """
    Vertically center the text in a specified window.

    ### arguments
     - `text`: The text to be vertically centered.
     - `size?`: The height of the centering window. If None, the terminal's height is used.

    ### returns
        The vertically centered text.
    """
    if size is None:
        size = get_terminal_size().lines

    linecount = len(text.splitlines())
    spaces = (size - linecount) // 2
    return '\n' * spaces + text + '\n' * spaces


def xcenter(text: AnyStringT,
            size: int | None = None,
            absolute: bool = False) -> AnyStringT:
    """
    Horizontally center the text in a specified window.

    ### arguments:
     - `text`: The text to be horizontally centered.
     - `size?`: The width of the centering window. If None, the terminal's width is used.
     - `absolute?`: If True, perform absolute centering for each line of the text.

    ### returns
        The horizontally centered text.
    """
    if size is None:
        size = get_terminal_size().columns

    if absolute:
        return '\n'.join(xcenter(line, size) for line in text.splitlines())

    maxline = max(
        (len(line) for line in rmansi(text).splitlines()),
        default=0
    )

    spaces = (size - maxline) // 2

    return '\n'.join((' ' * spaces) + line
                        for line in text.splitlines())


def center(text: AnyStringT,
           size: tuple[int | None, int | None] | None = None,
           absolute: bool = False) -> AnyStringT:
    """
    Center the text in a specified window. (X and Y)

    ### arguments:
     - `text`: The text to be centered.
     - `size?`: The size of the centering window. If None, the terminal's size is used.
     - `absolute?`: If True, perform absolute centering for each line of the text.

    ### returns
        The centered text.
    """
    if size is None:
        size = (None, None)
    return ycenter(xcenter(text, size[0], absolute), size[1])


from __future__ import annotations
from re import compile as regex_compile
from typing import overload, ClassVar

from .ansi import RESET, FG_COLOR
from .types import *


__all__ = ['Color', 'Fade', 'Col']


def rmansi(text: AnyStringT) -> AnyStringT:
    """
    Remove all the ANSI sequences in a text.

    ### arguments
     - `text`: The text from which ANSI sequences will be removed.

    ### returns
        The text with ANSI sequences removed.
    """
    regexp = regex_compile(r'(\x9B|\x1B\[)[0-?]*[ -\/]*[@-~]')
    return regexp.sub('', text)



class Color:
    rgb: tuple[int, int, int]
    """The RGB value of the color"""

    __slots__ = ('rgb',)

    def __init__(self,
                 r: int,
                 g: int,
                 b: int) -> None:
        """
        Create a color using RGB values.

        ### arguments
         - `r `: The red component of the RGB color.
         - `g `: The green component of the RGB color.
         - `b `: The blue component of the RGB color.
        """
        self.rgb = (r, g, b)


    def __repr__(self) -> str:
        return 'color({}, {}, {})'.format(*self.rgb)


    def __or__(self, color: Color) -> Color:
        return self.mix(color)


    @overload
    def __add__(self, other: Color) -> Fade:
        ...


    @overload
    def __add__(self, other: AnyStringT) -> AnyStringT:
        ...


    def __add__(self, other: AnyStringT | Color) -> AnyStringT | Fade:
        if isinstance(other, str):
            return self.colorate(other) + RESET

        elif isinstance(other, Color):
            return Fade([self, other])


    @property
    def hex(self) -> str:
        """The hexadecimal value of the color."""
        return bytes(self.rgb).hex()


    @property
    def ansi(self) -> str:
        return FG_COLOR.format(*map(str, self.rgb))


    @staticmethod
    def fromhex(hex: str) -> Color:
        """
        Create a color from it hexadecimal value.

        ### arguments
         - `hex`: The hexadecimal value of the color.

        ### returns
            The color.
        """
        if hex.startswith('#'):
            hex = hex[1:]
        return Color(*bytes.fromhex(hex))


    def mix(self, color: Color, scale: int = 50) -> Color:
        """
        Mix two colors together.

        ### arguments
         - `color`: The color to mix with the current one.
         -  `scale`: The mixing scale, affecting the ratio of the two colors.

        ### returns
            The new mixed color.
        """
        r = int(self.rgb[0] + (color.rgb[0] - self.rgb[0]) * scale / 100)
        g = int(self.rgb[1] + (color.rgb[1] - self.rgb[1]) * scale / 100)
        b = int(self.rgb[2] + (color.rgb[2] - self.rgb[2]) * scale / 100)
        return Color(r, g, b)


    def colorate(self, text: str) -> str:
        """
        Apply the color to a text.

        ### arguments
         - `text`: The text to which the color will be applied.

        ### returns
            The colored text.
        """
        return self.ansi + text



class Fade:
    colors: list[Color]
    """List of base colors used in the fade effect"""
    collength: int
    """The number of color transitions between each pair of base colors"""
    direction: FadeDirection
    """The direction of the fade when it is over a text"""

    __slots__ = ('colors', 'collength', 'direction')

    def __init__(self,
                 colors: list[Color],
                 collength: int = 10,
                 direction: FadeDirection = 'horizontal') -> None:
        """
        Initialize a Fade with specified colors.

        ### arguments
         - `colors `: List of base colors used in the fade effect.
         - `collength? `: The number of color transitions between each pair of base colors.
         - `direction? `: The direction of the fade when it is over a text.
        """
        self.colors = colors
        self.collength = collength
        self.direction = direction


    @overload
    def __add__(self, other: Color) -> Fade:
        ...


    @overload
    def __add__(self, other: AnyStringT) -> AnyStringT:
        ...


    def __add__(self, other: AnyStringT | Color) -> AnyStringT | Fade:
        if isinstance(other, str):
            return self.colorate(other) + RESET

        elif isinstance(other, Color):
            return Fade(self.colors + [other])


    def getcolors(self) -> list[Color]:
        """
        Generate a list of interpolated colors based on the colors and collength.

        ### returns
            A list of interpolated colors used in the fade.
        """
        colors: list[Color] = []

        for i in range(len(self.colors) - 1):
            start = self.colors[i]
            end = self.colors[i + 1]

            for j in range(self.collength):
                scale = int((j / self.collength) * 100)
                interpolated_color = start.mix(end, scale)
                colors.append(interpolated_color)

        return colors


    def colorate(self,
                 text: AnyStringT,
                 decal: int = 0) -> AnyStringT:
        """
        Apply the fade effect to a text string with optional starting offset.

        ### arguments
         - `decal`: The starting offset for the fade effect.

        ### returns
            The faded text.
        """
        text = rmansi(text)
        colored_text = ''
        colors = self.getcolors()
        colors += list(reversed(colors))
        colcount = len(colors)

        decal = decal % colcount

        for i in range(decal):
            colors.append(colors[0])
            del colors[0]

        if colcount == 0:
            return text

        match self.direction:
            case 'horizontal':
                for i, char in enumerate(text):
                    color = colors[i % colcount]
                    colored_text += color.ansi + char

            case 'vertical':
                for i, line in enumerate(text.splitlines()):
                    color = colors[i % colcount]
                    colored_text += color.ansi + line + '\n'

        return colored_text



class Col:
    red: ClassVar[Color]
    green: ClassVar[Color]
    blue: ClassVar[Color]
    black: ClassVar[Color]
    white: ClassVar[Color]
    gray: ClassVar[Color]
    grey: ClassVar[Color]
    darkgrey: ClassVar[Color]
    yellow: ClassVar[Color]
    orange: ClassVar[Color]
    purple: ClassVar[Color]
    cyan: ClassVar[Color]
    pink: ClassVar[Color]
    brown: ClassVar[Color]

    red = Color(255, 0, 0)
    green = Color(0, 255, 0)
    blue = Color(0, 0, 255)

    black = Color(0, 0, 0)
    white = Color(255, 255, 255)
    gray = grey = black | white
    darkgrey = grey | black

    yellow = Color(255, 255, 0)
    orange = Color(255, 165, 0)
    purple = Color(128, 0, 128)
    cyan = Color(0, 255, 255)
    pink = Color(245, 61, 177)
    brown = Color(165, 42, 42)


from .ansi import RESET
from .colors import Color, Fade
from .types import *



__all__ = ['Pattern']



class Pattern:
    pattern: dict[str, Color | Fade]
    """A dictionary mapping characters to Color or Fade"""

    __slots__ = ('pattern',)

    def __init__(self, pattern: dict[str, Color | Fade]) -> None:
        """
        Initialize a Pattern with the specified pattern.

        ### arguments:
         - pattern: A dictionary mapping characters to Color or Fade.
        """
        self.pattern = pattern


    def __add__(self, text: AnyStringT) -> AnyStringT:
        return self.colorate(text) + RESET


    def colorate(self,
                 text: AnyStringT,
                 decal: int = 0) -> AnyStringT:
        """
        Apply the pattern to a text.

        ### arguments:
         - text: The text to which the pattern will be applied.
         - decal: The starting offset for the pattern fades.

        ### returns:
            The input text string with the pattern applied.
        """
        colored_text = ''
        fadestates: dict[str, int] = {}

        for match in self.pattern.keys():
            for c in match:
                fadestates[c] = decal

        for char in text:
            color = None
            for match, ccolor in self.pattern.items():
                if char in match:
                    color = ccolor
                    break

            if not color:
                colored_text += RESET + char

            else:
                if isinstance(color, Color):
                    colored_text += color.colorate(char)

                elif isinstance(color, Fade):
                    colindex = fadestates[char]
                    fadecolors = color.getcolors()
                    fadecolors += list(reversed(fadecolors))

                    charcol = fadecolors[colindex % len(fadecolors)]
                    colored_text += charcol.colorate(char)

            for match, color in self.pattern.items():
                passed = ''
                if isinstance(color, Fade):
                    for c in match:
                        if c in passed:
                            continue

                        passed += c

                        if color.direction == 'horizontal':
                            if char == '\n':
                                fadestates[c] = decal
                            else:
                                fadestates[c] += 1

                        elif color.direction == 'vertical':
                            if char == '\n':
                                fadestates[c] += 1

        return colored_text


from time import sleep
from sys import stdout
from keyboard import add_hotkey, remove_hotkey
from threading import Event
from typing import ClassVar

from .colors import Color, Fade, Col
from .system import sys
from .ansi import RESET, UNDERLINE


__all__ = ['Icon', 'printd', 'inputd', 'choose', 'yesno', 'underline', 'Icons']



class Icon: # TODO
    symbol: str
    color: Color
    borders: tuple[str, str] | None
    borders_color: Color | None
    
    def __init__(self,
                 symbol: str,
                 color: Color,
                 borders: tuple[str, str] | None = None,
                 borders_color: Color | None = None) -> None:
        self.symbol = symbol
        self.color = color
        self.borders = borders
        self.borders_color = borders_color


    def __str__(self) -> str:
        return self.icon


    def __add__(self, other: str) -> str:
        return self.add(other)


    @property
    def icon(self) -> str:
        if self.borders:
            if self.borders_color:
                lborder = self.borders_color + self.borders[0]
                rborder = self.borders_color + self.borders[1]
            else:
                lborder = self.borders[0]
                rborder = self.borders[1]
        else:
            lborder = rborder = ''
        return lborder + (self.color + self.symbol) + rborder
    
    
    def add(self, text: str) -> str:
        return self.icon + ' ' + text
        



def printd(text: str,
           color: Color | Fade | None = None,
           interval: int | float = 0.05,
           lpadding: int = 0,
           linebreak: bool = True) -> None:
    """
    Prints text with a specified color or fade, and with an interval between each character.

    ### arguments
     - `text`: The text to be printed.
     - `color?`: The color or fade to apply to the text.
     - `interval? `: The time interval between printing each character.
     - `lpadding?`: The left padding margin of the text.
     - `linebreak?`: If True, adds a line break at the end of the printed text.
    """
    stdout.write(' ' * lpadding)
    interval = float(interval)

    if not interval:
        if color:
            stdout.write(color.colorate(text))
        else:
            stdout.write(text)
    else:
        if color:
            if isinstance(color, Color):
                stdout.write(color.ansi)
                for char in text:
                    stdout.write(char)
                    stdout.flush()
                    sleep(interval)

            elif isinstance(color, Fade):
                fade = Fade(color.colors, color.collength, color.direction)
                fadecolors = fade.getcolors()
                fadecolors += list(reversed(fadecolors))
                colcount = len(fadecolors)

                for i, char in enumerate(text):
                    fadecol = fadecolors[i % colcount]
                    stdout.write(fadecol.colorate(char))
                    stdout.flush()
                    sleep(interval)
        else:
            for char in text:
                stdout.write(char)
                stdout.flush()

    if linebreak:
        stdout.write('\n')
    stdout.write(RESET)


def inputd(prompt: str,
           color: Color | Fade,
           resp_color: Color | None = None,
           interval: int | float = 0.05,
           lpadding: int = 0,
           cursor: bool = True) -> str:
    """
    Prompts the user for input with a specified color or fade effect and returns the user's response.

    ### arguments
     - `prompt`: The prompt message to display.
     - `color`: The color or fade for the prompt message.
     - `resp_color?`: The color for the user's response or None for no color.
     - `interval? `: The time interval between printing each character.
     - `lpadding?`: The left padding margin of the text.
     - `cursor?`: If True, displays the cursor during user input.

    ### returns
        The response of the user.
    """
    cursor_state = sys.cursor
    printd(prompt, color, interval, lpadding, False)

    if resp_color:
        stdout.write(resp_color.ansi)

    sys.cursor = cursor
    result = input()
    sys.cursor = cursor_state

    stdout.write(RESET)
    return result


def choose(prompt: str,
           choices: list[str],
           over_color: Color | Fade | None = None,
           lpadding: int = 0) -> int:
    """
    Ask the user to choose one of many choices.

    ### arguments
     - `prompt`: The prompt message to display.
     - `choices`: The choices to display.
     - `over_color?`: The color of the choice that the cursor is on.
     - `lpadding?`: The left padding margin.

    ### returns
        The response of the user exprimed in boolean.
    """
    cursor_state = sys.cursor
    sys.cursor = False

    if not over_color:
        over_color = Fade([(Col.blue | Col.white), (Col.purple | Col.white)], 20)

    lpad = ' ' * lpadding
    choices_lpad = ' ' * (lpadding + 2)

    print(lpad + (Icons.interrogation + prompt))
    choosed = 0
    continu = Event()

    choice_prefix = Col.darkgrey + '[ {} ' + (Col.darkgrey + '] ')

    def up():
        nonlocal choosed
        choosed -= 1
        if choosed < 0:
            choosed = len(choices) - 1
        refresh_choices()

    def down():
        nonlocal choosed
        choosed += 1
        if choosed >= len(choices):
            choosed = 0
        refresh_choices()

    def send():
        nonlocal continu
        continu.set()

    def refresh_choices():
        display_choices('\x1b[1A\x1b[2K' * len(choices))

    def display_choices(initbuffer: str = ''):
        result = initbuffer
        for i, choice in enumerate(choices):
            num = str(i + 1)
            if i == choosed:
                text = choice_prefix.format(over_color + num) + \
                    (over_color + choice) + (Col.darkgrey + ' «')
            else:
                text = choice_prefix.format(RESET + num) + choice

            result += choices_lpad + text + '\n'

        stdout.write(result)

    add_hotkey('up', up)
    add_hotkey('down', down)
    add_hotkey('enter', send)
    display_choices('')

    continu.wait()
    remove_hotkey(up)
    remove_hotkey(down)
    remove_hotkey(send)

    sys.cursor = cursor_state
    return choosed


def yesno(prompt: str,
          over_color: Color | Fade | None = None,
          default: bool = True,
          lpadding: int = 0) -> int:
    """
    Ask the user to choose between yes and no.

    ### arguments
     - `prompt`: The prompt message to display.
     - `over_color?`: The color of the choice that the cursor is on.
     - `default?`: The default choice.
     - `lpadding?`: The left padding margin.

    ### returns
        The index of the item choosen.
    """
    cursor_state = sys.cursor
    sys.cursor = False

    if not over_color:
        over_color = Col.blue | Col.white

    prefix = '\r' + ' ' * lpadding + (Icons.interrogation + prompt) + ' '
    response = default
    continu = Event()

    def display_choice():
        if response:
            yes = over_color + underline('Yes')
            no = 'No'
        else:
            yes = 'Yes'
            no = over_color + underline('No')
        stdout.write(prefix + yes + (Col.gray + ' / ') + no)
        
    def change():
        nonlocal response
        response = not response
        display_choice()

    def send():
        nonlocal continu
        continu.set()

    add_hotkey('left', change)
    add_hotkey('right', change)
    add_hotkey('enter', send)
    display_choice()

    continu.wait()
    remove_hotkey(change)
    remove_hotkey(send)

    stdout.write('\n')
    sys.cursor = cursor_state
    return response


def underline(text: str) -> str:
    """
    Underline a text.

    ### arguments
     - `text`: The text to underline.

    ### returns
        The text underlined.
    """
    return UNDERLINE + text + RESET



class Icons:
    exclamation: ClassVar[Icon]
    interrogation: ClassVar[Icon]

    exclamation = Icon('!', Col.red.mix(Col.white, 30))
    interrogation = Icon('?', Col.blue | Col.white)


from typing import TypeVar, Literal, TypeAlias, LiteralString


__all__ = ['AnyStringT', 'FadeDirection']


AnyStringT = TypeVar('AnyStringT', str, LiteralString)
FadeDirection: TypeAlias = Literal['horizontal', 'vertical']